part of jaguar_jwt.src;

/// Holds configuration to issue JWT token during authentication phase
class AuthenticationConf {
  /// Issuer of the JWT token
  ///
  /// Fills `iss` field in JWT token
  final String issuer;

  /// Allowed audience
  ///
  /// Fills `aud` field in JWT token
  final List<String> audience;

  /// The signature context to sign the JWT token with
  final JwaSignatureContext signatureContext;

  /// Duration for which the issued token is valid
  final Duration tokenDuration;

  const AuthenticationConf(this.signatureContext, this.issuer,
      {this.audience: const <String>[],
      this.tokenDuration: const Duration(days: 1)});

  /// Creates Authentication configuration with symmetric key encryption
  factory AuthenticationConf.SymmetricKey(String symmetricKey, String issuer,
      {List<String> audience: const <String>[],
      Duration tokenDuration: const Duration(days: 1)}) {
    final signatureContext = new JwaSymmetricKeySignatureContext(symmetricKey);
    return new AuthenticationConf(signatureContext, issuer,
        audience: audience, tokenDuration: tokenDuration);
  }

  /// Encodes the given claim set into JWT token `String`
  String encodeClaimSet(ClaimSet claimSet) {
    JsonWebToken jwt = new JsonWebToken.jws(claimSet, signatureContext);
    return jwt.encode();
  }

  /// Encodes for the given subject
  String encodeForSubject(String subject,
      {Map<String, dynamic> payload: const {}, DateTime issuedAt}) {
    if (issuedAt == null) {
      issuedAt = new DateTime.now();
    }
    DateTime expiry = issuedAt.add(tokenDuration);
    ClaimSet claimSet = new ClaimSet(subject, issuer, audience, payload,
        expiry: expiry, issuedAt: issuedAt);
    return encodeClaimSet(claimSet);
  }
}

/// Holds configuration to decode and validate JWT token during authorization
/// phase
class AuthorizationConf {
  /// The allowed/expected issuer
  ///
  /// The issuer of the incoming JWT token must match [issuer] for authorization
  /// to succeed
  final String issuer;

  /// Allowed audience. Incoming JWT token must contain at least one of these
  /// audiences for the authorization to succeed
  final List<String> audience;

  /// Signature context for JWT encryption
  ///
  /// Used to authorize the JWT token
  final JwaSignatureContext signatureContext;

  const AuthorizationConf(this.signatureContext, this.issuer,
      {this.audience: const <String>[]});

  factory AuthorizationConf.SymmetricKey(String symmetricKey, String issuer,
      {List<String> audience: const <String>[]}) {
    final signatureContext = new JwaSymmetricKeySignatureContext(symmetricKey);
    return new AuthorizationConf(signatureContext, issuer, audience: audience);
  }

  /// Creates [JaguarJwtValidationContext]
  JaguarJwtValidationContext validationContext() {
    return new JaguarJwtValidationContext(
        signatureContext, new ClaimSetValidationContext(issuer, audience));
  }

  /// Decodes the given token
  JsonWebToken decodeToken(String token) =>
      JaguarJwtCodec.decodeToken(token, validationContext());

  /// Validates token
  bool isTokenValid(String token) =>
      JaguarJwtCodec.isTokenValid(token, validationContext());

  /// Validates claim set
  bool isClaimSetValid(ClaimSet claimSet) =>
      JaguarJwtCodec.isClaimSetValid(claimSet, validationContext());
}

/// Encodes, decodes and validates JWT token
class JaguarJwtCodec {
  /// Issues custom token
  static String encodeToken(
      ClaimSet claimSet, JwaSignatureContext signatureContext) {
    JsonWebToken jwt = new JsonWebToken.jws(claimSet, signatureContext);
    return jwt.encode();
  }

  /// Decodes the given token
  static JsonWebToken decodeToken(
          String token, JaguarJwtValidationContext validationContext) =>
      new JsonWebToken.decode(token,
          validationContext: validationContext, claimSetParser: ClaimSet.parse);

  /// Validates token against given [validationContext]
  static bool isTokenValid(
      String token, JaguarJwtValidationContext validationContext) {
    if (token == null) {
      return false;
    }

    JsonWebToken jwt = decodeToken(token, validationContext);

    return isClaimSetValid(jwt.claimSet, validationContext);
  }

  /// Validates claim set against given [validationContext]
  static bool isClaimSetValid(
      ClaimSet claimSet, JaguarJwtValidationContext validationContext) {
    Set<ConstraintViolation> violations =
        claimSet.validate(validationContext.claimSetValidationContext);

    return violations.isEmpty;
  }
}
